#!/usr/bin/ruby

require "mistfiles"
require "net/https"
require "uri"
require "date"
require "digest/md5"

class MistFiles
	AuthenticationError = Class.new(StandardError)
  RACKSPACE_US_URL = "https://identity.api.rackspacecloud.com/"
  RACKSPACE_UK_URL = "https://lon.identity.api.rackspacecloud.com/"
  
  def authenticated?
    @token && @token.length > 0 && @expires && @expires.to_time.to_i > Time.now.to_i
  end
  
  def authenticate!
    json_body = {
      :auth => {
        "RAX-KSKEY:apiKeyCredentials" => {
          :username => @username,
          :apiKey => @api_key
        }
      }
    }
    
    res = send_request(
      :host => @british ? RACKSPACE_UK_URL : RACKSPACE_US_URL,
      :path => "/v2.0/tokens",
      :method => :post,
      :body_type => :json,
      :params => json_body
    )
		
		raise AuthenticationError, "HTTP error #{res.code}" if res.code < 200 || res.code > 299
    
    body = JSON.parse(res.body)
    raise AuthenticationError, "#{self.class}: Invalid response data, something must be wrong." unless body.member?("access")
    access = body["access"]

    @token = access["token"]["id"]
    @expires = DateTime.parse(access["token"]["expires"])
    @user = access["user"]
    @user[:id] = @user.delete "id"
    @user[:username] = @user.delete "name"
    @user[:default_region] = @user.delete("RAX-AUTH:defaultRegion").downcase.to_sym
    
    catalog = access["serviceCatalog"].select { |service| RS_SERVICE_TYPES.include?(service["type"]) }.each_with_object({}) { |entry,ctlg| ctlg[entry["type"]] = entry["endpoints"] }
    
    @cdn_catalog = Hash[catalog["rax:object-cdn"].map{ |endpt| [endpt["region"].downcase.to_sym, endpt["publicURL"]] }]
    @files_catalog = Hash[catalog["object-store"].map{ |endpt| [endpt["region"].downcase.to_sym, endpt[@internal ? "internalURL" : "publicURL"]] }]
    nil
  end
  
  def auth_if_necessary!
    authenticate! unless authenticated?
  end
end