#!/usr/bin/env

require "mistfiles"

class MistFiles
  def create_container(params={})
		auth_if_necessary!
    container = params[:container] || params["container"]
    headers = params[:headers] || params["headers"]
    send_request(
      :host => @files_catalog[@region],
      :path => "/#{container}",
      :method => :put,
      :params => params || {},
      :headers => headers
    )
  end
  
  def update_container(params={})
		auth_if_necessary!
    container = params[:container] || params["container"]
    headers = params[:headers] || params["headers"]
    send_request(
      :host => @files_catalog[@region],
      :path => "/#{container}",
      :method => :post,
      :params => params || {},
      :headers => headers
    )
  end
  
  def destroy_container(params={})
		auth_if_necessary!
    container = params[:container] || params["container"]
    send_request(
      :host => @files_catalog[@region],
      :path => "/#{container}",
      :method => :delete,
      :params => params || {}
    )
  end
  
  # list_containers(params={})
  # :limit => the number of containers to return
  # :marker => returns results after the container name specified here
  # :end_marker => retuns results before the container name
  def list_containers(params={})
    auth_if_necessary!
    send_request(
      :host => @files_catalog[@region],
      :method => :get,
      :params => params || {}
    )
  end
  
  # lists objects
  # ALSO gets container metadata
  def list_objects(params={})
    auth_if_necessary!
    
    container = (params[:container] || params["container"])
    
    send_request(
      :host => @files_catalog[@region],
      :path => "/#{container}",
      :method => :get,
      :params => params || {}
    )
  end
  
  # parameters
  # :ttl => The ttl in seconds
  # :enabled => A true or false value indicating the CDN status of the container
  # :log_retention => Whether RackSpace should store CDN access logs in the container.
  # IMPORTANT: if the log_retention parameter is passed, this request will not be able to 
  def cdn_settings=(params={})
		auth_if_necessary!
    container = params[:container] || params["container"]
    ttl = (params[:ttl] || params["ttl"]).to_i.to_s
    enabled = (params[:enabled] || params["enabled"]) ? "True" : "False"
    log_retention = params[:log_retention] || params["log_retention"]
    
    headers = {}
    headers["X-Ttl"] = ttl unless ttl.nil? || ttl.empty?
    headers["X-Cdn-Enabled"] = container unless container.nil? || container.empty?
    headers["X-Log-Retention"] = log_retention if params.member?(:log_retention) || params.member?("log_retention")
    
    method = headers.member?("X-Log-Retention") ? :post : :put
    
    send_request(
      :host => @cdn_catalog[@region],
      :path => "/#{container}",
      :method => method,
      :headers => headers
    )
  end
  
  def cdn_settings(params={})
		auth_if_necessary!
    container = params[:container] || params["container"]
    send_request(
      :host => @cdn_catalog[@region],
      :path => "/#{container}",
      :method => :head
    )
  end
end