#!/usr/bin/ruby

require "mistfiles"
require "net/https"
require "uri"
require "json"

class MistFilesResponse
  attr_accessor :code, :headers, :body
  
  # takes a Net::HTTPResponse
  def initialize(opts={})
		@res = opts[:response]
    @json = opts[:parse_json]
    @code = @res.code.to_i
    @headers = Hash[@res.to_hash.map{ |k,v| [k,(v.first rescue v)] }]
		@body = @res.body || ""
		@body = JSON.parse @res.body if @json && !@body.empty?
  end
  
  def to_rack
    [@code, @headers, (@json ? @body.to_json : @body)]
  end
end

class MistFiles
  RS_SERVICE_TYPES = ["object-store", "rax:object-cdn"]
  VERB_MAP = {
    :get    => Net::HTTP::Get,
    :post   => Net::HTTP::Post,
    :put    => Net::HTTP::Put,
    :patch  => Net::HTTP::Patch,
    :delete => Net::HTTP::Delete,
    :head   => Net::HTTP::Head
  }
  DEFAULT_HTTP_HEADERS = {
    "Accept" => "application/json",
    "Content-Type" => "application/json"
  }
  
  def http(opts={})
    @http_objs ||= {}
    
    uri = opts[:uri] || opts["uri"]
    http = @http_objs[uri.host]
    return http unless http.nil?
    
    http = Net::HTTP.new uri.host, uri.port
    if uri.port == 443
      http.use_ssl = true
      http.verify_mode = OpenSSL::SSL::VERIFY_PEER
    end
    @http_objs[uri.host] = http
    http
  end
  
  def send_request(opts={})
    parse_json  = (opts[:parse_json] || opts["parse_json"]) == true
    host        = opts[:host] || opts["host"]
    path        = opts[:path] || opts["path"] || ""
    method      = (opts[:method] || opts["method"]).to_sym
    params      = opts[:params] || opts["params"] || {}
    body        = opts[:body] || opts["body"]
    body_type   = (opts[:body_type] || opts["body_type"] || :ignored).to_sym
    headers     = DEFAULT_HTTP_HEADERS.merge(opts[:headers] || opts["headers"] || {})
    headers.merge!({ "X-Auth-Token" => @token }) unless @token.nil? || @token.empty?
    
    uri = URI.parse host
    path = uri.path + path
    
    request = 
    case method
    when :get
      path << "?" + URI.encode_www_form(params) unless params.empty?
      VERB_MAP[method].new path
    else
      request = VERB_MAP[method].new path
      case body_type
      when :json
        request.body = 
        case params
        when String
          params
        else
          params.to_json
        end
      when :form_data
        request.set_form_data params
      when :raw
        request.body = body
      end
      request
    end
    
    headers.each { |k,v| request.add_field k,v }

    MistFilesResponse.new(
			:response => http(:uri => uri).request(request),
			:parse_json => parse_json
		)
  end
end